# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '26/1/22'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtCore import Qt, QPoint, Signal, QMimeData
from PySide6.QtGui import QColor, QDrag, QPixmap, QPainter, QPen
from PySide6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QScrollArea, QSizePolicy

from ..yanBolme.dikeyEtiket import DikeyEtiket


#######################################################################
class BaslikWidget(QWidget):
    sol_tiklandi = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, kapatilabilir_mi=False, parent=None):
        super(BaslikWidget, self).__init__(parent)
        self.solClickKucultBuyult = False

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        super(BaslikWidget, self).mousePressEvent(event)
        if event.button() == Qt.MouseButton.LeftButton:
            self.solClickKucultBuyult = True
            self.mGPos = event.globalPosition()

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        super(BaslikWidget, self).mouseReleaseEvent(event)
        if self.solClickKucultBuyult:
            hareket = event.globalPosition().toPoint() - self.mGPos.toPoint()
            if hareket.manhattanLength() < 3:
                self.sol_tiklandi.emit()
                # event.ignore()
                # return
            self.solClickKucultBuyult = False


#######################################################################
class YuzenWidget(QWidget):
    kenaraAlVeyaKapatTiklandi = Signal(QWidget, bool)

    # ---------------------------------------------------------------------
    def __init__(self, kapatilabilir_mi=False, parent=None):
        super(YuzenWidget, self).__init__(parent)

        self.kapatilabilir_mi = kapatilabilir_mi
        self.parkPos = QPoint(0, 100)
        self.eskiAcikPos = QPoint(0, 100)
        self.eskiYuzenPos = QPoint(0, 0)
        self.eskiSize = self.size()
        self.setMinimumSize(100, 100)
        self.eskiMinimumSize = self.minimumSize()

        self.kucuk_mu = False
        self.dikey_mi = False
        self.yuzerken_dikey_miydi = False
        # ilk acilista True olmasi
        self.cubukta_mi = False
        self.sol_cubukta_mi = True
        self.sira = 0

        self.dugme = None

        self.renkYazi = QColor(255, 255, 255)
        # self.renkArkaplan = QColor(200, 205, 210)
        self.renkArkaplan = QColor(153, 170, 187)

        # self.setMinimumHeight(400)
        self.setAutoFillBackground(True)
        # self.setStyleSheet("QWidget {background-color: #ccc;}")
        self.setContentsMargins(0, 0, 0, 0)
        # ~ self.setWindowFlags(Qt.FramelessWindowHint|Qt.NoDropShadowWindowHint| Qt.Tool)
        # self.setWindowFlags(Qt.FramelessWindowHint | Qt.Tool | Qt.WindowType.WindowStaysOnTopHint)
        # Tool olunca pencere disina da tasabiliyoır ve pencere ile hareket etmiyor.
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)
        # self.setSizeGripEnabled(True)
        self.setStatusTip(self.tr("Move with left mouse button, resize with right mouse button."))
        # self.setToolTip(self.tr("Move with left mouse button, resize with right mouse button."))

        self.setFocusPolicy(Qt.FocusPolicy.TabFocus)

        self.mPos = QPoint()
        self.sagClick = False
        self.solClick = False
        self.solClickKucultBuyult = False

        self.baslikWidget = BaslikWidget(self)
        self.baslikWidget.sol_tiklandi.connect(self.kucult_buyult)
        # self.baslikWidget.setFixedHeight(20)
        self.baslikWidget.setContentsMargins(0, 0, 0, 0)
        self.baslikWidget.setAutoFillBackground(True)

        p = self.baslikWidget.palette()
        p.setColor(self.baslikWidget.foregroundRole(), self.renkYazi)
        p.setColor(self.baslikWidget.backgroundRole(), self.renkArkaplan)
        self.baslikWidget.setPalette(p)

        self.baslikEtiket = DikeyEtiket(self.baslikWidget, dikey=False)
        self.baslikEtiket.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.btnKucult = QPushButton("<", self.baslikWidget)
        self.btnKucult.setFixedWidth(20)
        self.btnKucult.setFlat(True)
        self.btnKucult.setAutoFillBackground(True)
        self.btnKucult.clicked.connect(self.kucult_buyult)

        p = self.btnKucult.palette()
        p.setColor(self.btnKucult.foregroundRole(), self.renkYazi)
        p.setColor(self.btnKucult.backgroundRole(), self.renkArkaplan)
        self.btnKucult.setPalette(p)

        self.btnDondur = QPushButton("o", self.baslikWidget)
        self.btnDondur.setFixedWidth(20)
        self.btnDondur.setFlat(True)
        self.btnDondur.setAutoFillBackground(True)
        self.btnDondur.clicked.connect(lambda: self.dondur(not self.dikey_mi))
        # self.btnDondur.hide()

        p = self.btnDondur.palette()
        p.setColor(self.btnDondur.foregroundRole(), self.renkYazi)
        p.setColor(self.btnDondur.backgroundRole(), self.renkArkaplan)
        self.btnDondur.setPalette(p)

        if self.kapatilabilir_mi:
            yazi = "x"
        else:
            yazi = "#"
        self.btnKenaraAlVeyaKapat = QPushButton(yazi, self.baslikWidget)
        self.btnKenaraAlVeyaKapat.setFixedWidth(20)
        self.btnKenaraAlVeyaKapat.setFlat(True)
        self.btnKenaraAlVeyaKapat.setAutoFillBackground(True)
        self.btnKenaraAlVeyaKapat.clicked.connect(self.kenara_al_veya_kapat)

        p = self.btnKenaraAlVeyaKapat.palette()
        p.setColor(self.btnKenaraAlVeyaKapat.foregroundRole(), self.renkYazi)
        p.setColor(self.btnKenaraAlVeyaKapat.backgroundRole(), self.renkArkaplan)
        self.btnKenaraAlVeyaKapat.setPalette(p)

        self.anaLay = QVBoxLayout(self)
        self.anaLay.setContentsMargins(0, 0, 0, 0)
        self.anaLay.setSpacing(0)
        # self.anaLay.setSizeConstraint(QVBoxLayout.SetFixedSize)

        self.layBaslik = QHBoxLayout()
        self.layBaslik.setContentsMargins(0, 0, 0, 0)
        # self.layBaslik.setSizeConstraint(QVBoxLayout.SizeConstraint.SetMaximumSize)

        # lay.addStretch()
        # 2 dugme icin 36
        # lay.addSpacing(36)
        # 1 dugme icin 16
        # lay.addSpacing(16)
        self.layBaslik.addWidget(self.btnKenaraAlVeyaKapat)
        self.layBaslik.addWidget(self.btnDondur)
        self.layBaslik.addWidget(self.baslikEtiket)
        self.layBaslik.addWidget(self.btnKucult)

        self.baslikWidget.setLayout(self.layBaslik)

        self.anaLay.addWidget(self.baslikWidget)

        self.icerikScroll = QScrollArea()
        self.icerikScroll.setContentsMargins(0, 0, 0, 0)
        # icerikScroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        # icerikScroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.icerikScroll.setWidgetResizable(True)
        # scroll.setLayout(self.anaLay)
        self.anaLay.addWidget(self.icerikScroll)

        self.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)

    # ---------------------------------------------------------------------
    def yazBaslik(self, yazi):
        self.baslikEtiket.setText(yazi)

    # ---------------------------------------------------------------------
    def restoreGeometry(self, geo):
        super(YuzenWidget, self).restoreGeometry(geo)
        self.eskiSize = self.size()

    # ---------------------------------------------------------------------
    def ekleWidget(self, widget):
        # self.resize(widget.size())

        # gripper = QSizeGrip(widget)
        # l = QHBoxLayout(widget)
        #
        # l.setContentsMargins(0, 0, 0, 0)
        # l.addWidget(gripper, 0, Qt.AlignRight | Qt.AlignBottom)

        self.icerikScroll.setWidget(widget)
        # self.anaLay.addWidget(widget)

        # self.adjustSize()

    # # ---------------------------------------------------------------------
    # def sizeHint(self):
    #     # return self.baslikWidget.size() + self.icerikScroll.size()
    #
    #     if self.kucuk_mu:
    #         return self.baslikWidget.size()
    #     if self.dikey_mi:
    #         return self.icerikScroll.size() + QSize(20, 0)
    #     else:
    #         return self.icerikScroll.size() + QSize(0, 20)

    # # ---------------------------------------------------------------------
    # def resizeEvent(self, event):
    #
    #     super(YuzenWidget, self).resizeEvent(event)

    # ---------------------------------------------------------------------
    def kaydetme_bilgisi(self):
        return f"{int(self.dikey_mi)}" \
               f"{int(self.kucuk_mu)}" \
               f"{int(self.cubukta_mi)}" \
               f"{int(self.sol_cubukta_mi)}" \
               f"{self.sira}"

    # ---------------------------------------------------------------------
    def yukleme_bilgisi(self, bilgi):
        dikey_mi = bool(int(bilgi[0]))
        self.kucuk_mu = bool(int(bilgi[1]))
        self.cubukta_mi = bool(int(bilgi[2]))
        self.sol_cubukta_mi = bool(int(bilgi[3]))
        self.sira = int(bilgi[4])

        if self.cubukta_mi:
            # kenaraAlVeyaKapatTiklandi asil emit edildigi yer burasi degil
            # simdilik mekanizmadan faydalaniyoruz, acilista, kaydedilmis bilgiler
            # okunup ywler burda guncellenince, dugmeye tiklanmis gibi eski yerlerine yerlestiriyoruz
            self.kenaraAlVeyaKapatTiklandi.emit(self, True)
        else:
            # self.dondur(dikey_dondur=dikey_mi)
            if dikey_mi:
                self.dikey_dondur()

        # degiskene bilgi girildi ama daha kucultulup buyutulmedigi icin terslemiyoruz degiskeni
        if self.kucuk_mu:
            self.kucult(ilk_acilis_mi=True)
        else:
            self.buyult(ilk_acilis_mi=True)

    # ---------------------------------------------------------------------
    def setMinimumWidthVeEskiMinimumSize(self, width):
        # metod override ozellikle yapilmadi
        self.setMinimumWidth(width)
        self.eskiMinimumSize = self.minimumSize()

    # ---------------------------------------------------------------------
    def moveEvent(self, event):
        if self.kucuk_mu:
            # self.parkPos = event.oldPos()
            self.parkPos = event.pos()
        else:
            self.eskiAcikPos = event.pos()
        super(YuzenWidget, self).moveEvent(event)

    # ---------------------------------------------------------------------
    def sola_yasla(self):
        self.layBaslik.setDirection(QHBoxLayout.Direction.LeftToRight)
        self.btnKucult.setText("<")
        self.sol_cubukta_mi = True

    # ---------------------------------------------------------------------
    def saga_yasla(self):
        self.layBaslik.setDirection(QHBoxLayout.Direction.RightToLeft)
        self.btnKucult.setText(">")
        self.sol_cubukta_mi = False

    # ---------------------------------------------------------------------
    def cubuga_tasindi_yuklenirken(self, solSag):
        self.eskiYuzenPos = self.pos()
        self.eskiSize = self.size()

        if solSag == "sol":
            self.sola_yasla()
        elif solSag == "sag":
            self.saga_yasla()

        # self.baslikWidget.hide()
        self.btnDondur.hide()
        # self.btnKenaraAlVeyaKapat.hide()

    # ---------------------------------------------------------------------
    def cubuga_tasindi(self, solSag):

        self.eskiYuzenPos = self.pos()
        self.eskiSize = self.size()
        self.dikey_mi = self.baslikEtiket.dikey

        if self.dikey_mi:
            self.yuzerken_dikey_miydi = True
            self.yatay_dondur()
        else:
            self.yuzerken_dikey_miydi = False

        if solSag == "sol":
            self.sola_yasla()
        elif solSag == "sag":
            self.saga_yasla()

        # self.baslikWidget.hide()
        if self.kucuk_mu:
            self.buyult()
        self.btnDondur.hide()
        # # self.btnKucult.hide()
        # self.btnKenaraAlVeyaKapat.hide()
        self.cubukta_mi = True

    # ---------------------------------------------------------------------
    def cubuktan_atildi(self):
        self.baslikWidget.show()
        self.btnDondur.show()
        # # self.btnKucult.show()
        # self.btnKenaraAlVeyaKapat.show()

        if self.yuzerken_dikey_miydi:
            self.dikey_dondur()

        self.cubukta_mi = False
        self.move(self.eskiYuzenPos)
        self.resize(self.eskiSize)
        self.adjustSize()

    # ---------------------------------------------------------------------
    def kenara_al_veya_kapat(self):

        if self.kapatilabilir_mi:
            self.hide()
        else:
            # kenara aliyoruz
            # self.btnKenaraAlVeyaKapat.hide()
            # self.kucult()
            # self.move(self.parkPos)

            self.kenaraAlVeyaKapatTiklandi.emit(self, False)

    # ---------------------------------------------------------------------
    def dondur(self, dikey_dondur):
        # self.icerikScroll.setVisible(not self.icerikScroll.isVisible())

        if dikey_dondur:
            self.dikey_dondur()
        else:
            self.yatay_dondur()

    # ---------------------------------------------------------------------
    def dikey_dondur(self):
        self.setMinimumWidth(20)
        self.dikey_mi = True
        self.baslikEtiket.dikey = True
        self.baslikEtiket.setFixedSize(16777215, 16777215)
        self.baslikEtiket.resize(self.baslikEtiket.size().transposed())
        self.baslikEtiket.setFixedWidth(20)
        self.baslikEtiket.update()
        self.baslikWidget.setFixedSize(16777215, 16777215)
        self.baslikWidget.resize(self.baslikWidget.size().transposed())
        self.baslikWidget.setFixedWidth(20)
        self.baslikWidget.update()
        self.layBaslik.setDirection(QVBoxLayout.Direction.TopToBottom)
        if self.sol_cubukta_mi:
            self.anaLay.setDirection(QVBoxLayout.Direction.LeftToRight)
        else:
            self.anaLay.setDirection(QVBoxLayout.Direction.RightToLeft)
        self.adjustSize()
        # self.resize(self.size().transposed())

    # ---------------------------------------------------------------------
    def yatay_dondur(self):
        self.setMinimumHeight(20)
        self.dikey_mi = False
        self.baslikEtiket.dikey = False
        self.baslikEtiket.setFixedSize(16777215, 16777215)
        self.baslikEtiket.resize(self.baslikEtiket.size().transposed())
        self.baslikEtiket.setFixedHeight(20)
        self.baslikEtiket.update()
        self.baslikWidget.setFixedSize(16777215, 16777215)
        # self.baslikWidget.setFixedSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX)
        self.baslikWidget.resize(self.baslikWidget.size().transposed())
        self.baslikWidget.setFixedHeight(20)
        self.baslikWidget.update()
        self.layBaslik.setDirection(QVBoxLayout.Direction.LeftToRight)
        self.anaLay.setDirection(QVBoxLayout.Direction.TopToBottom)

        self.adjustSize()
        # self.resize(self.size().transposed())

    # ---------------------------------------------------------------------
    def kucult(self, ilk_acilis_mi=False):
        # if self.baslikEtiket.dikey:
        #     if self.anaLay.direction() == QVBoxLayout.Direction.RightToLeft:
        #         self.move(self.pos() + QPoint(self.icerikScroll.rect().width(), 0))

        self.kucuk_mu = True
        if self.cubukta_mi:
            self.hide()
            self.dugme.renk_degistir(acik_mi=False)
            # if not self.dikey_mi:
            #     self.dikey_dondur()
        else:
            self.eskiSize = self.size()
            self.icerikScroll.hide()
            # yuzuyor olsa da hangi cubukta dugmesi var
            if self.sol_cubukta_mi:
                self.btnKucult.setText(">")
            else:
                if self.dikey_mi:
                    self.btnKucult.setText("<")
                    if not ilk_acilis_mi:
                        self.move(self.pos() + QPoint(self.width() - 20, 0))
                else:
                    self.btnKucult.setText(">")

            if self.dikey_mi:
                self.setMinimumWidth(20)
            else:
                self.setMinimumHeight(20)
            self.adjustSize()
            # self.resize(self.baslikWidget.size())

    # ---------------------------------------------------------------------
    def buyult(self, ilk_acilis_mi=False):
        # if self.dikey_mi:
        #     if (self.pos().x() + self.icerikScroll.rect().width()) < self.parent().rect().width():
        #         self.anaLay.setDirection(QVBoxLayout.Direction.RightToLeft)
        #     else:
        #         self.anaLay.setDirection(QVBoxLayout.Direction.LeftToRight)
        #
        #     if self.anaLay.direction() == QVBoxLayout.Direction.RightToLeft:
        #         self.move(self.pos() + self.mapToParent(self.rect().topLeft()))
        #
        # else:
        #     if self.pos().y() + self.icerikScroll.rect().height() < self.parent().rect().height():
        #         self.anaLay.setDirection(QVBoxLayout.Direction.TopToBottom)
        #     else:
        #         self.anaLay.setDirection(QVBoxLayout.Direction.BottomToTop)

        self.kucuk_mu = False
        self.setMinimumSize(self.eskiMinimumSize)
        if self.cubukta_mi:
            self.show()
            self.dugme.renk_degistir(acik_mi=True)
        else:
            self.icerikScroll.show()
            self.resize(self.eskiSize)
            self.adjustSize()
            # yuzuyor olsa da hangi cubukta dugmesi var
            if self.sol_cubukta_mi:
                self.btnKucult.setText("<")
            else:
                if self.dikey_mi:
                    self.btnKucult.setText(">")
                    if not ilk_acilis_mi:
                        self.move(self.pos() - QPoint(self.icerikScroll.width(), 0))
                else:
                    self.btnKucult.setText("<")
            # self.btnKucult.setText("<")

    # ---------------------------------------------------------------------
    def kucult_buyult(self):

        if self.kucuk_mu:
            self.buyult()
        else:
            self.kucult()

    # # ---------------------------------------------------------------------
    # def mouseDoubleClickEvent(self, event):
    #
    #     self.kenara_al_veya_kapat()
    #     # self.kucult_buyult()
    #
    #     # if self.cubukta_mi:
    #     #     self.kenara_al_veya_kapat()
    #     # else:
    #     #     self.kucult_buyult()
    #     super(YuzenWidget, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        super(YuzenWidget, self).mousePressEvent(event)

        if not self.cubukta_mi:
            self.raise_()

            if event.button() == Qt.MouseButton.LeftButton:
                self.mPos = event.pos()
                self.solClick = True
                self.setCursor(Qt.CursorShape.ClosedHandCursor)

            if event.button() == Qt.MouseButton.RightButton and self.icerikScroll.isVisible():
                self.sagDragx = event.x()
                self.sagDragy = event.y()
                self.enSagX = self.width()
                self.enAltY = self.height()
                self.sagClick = True
                self.setCursor(Qt.CursorShape.SizeAllCursor)

    # ---------------------------------------------------------------------
    def sag_kenara_yanastir(self):
        pr = self.parent().rect()
        self.move(pr.right() - self.width() - 9, pr.top() + 30)

    # ---------------------------------------------------------------------
    def sol_kenara_yanastir(self):
        pr = self.parent().rect()
        self.move(10, pr.top() + 30)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        super(YuzenWidget, self).mousePressEvent(event)
        if not self.cubukta_mi:
            if event.buttons() & Qt.MouseButton.LeftButton:
                if self.solClick:
                    fark = event.pos() - self.mPos
                    yeniPos = self.pos() + fark
                    pr = self.parent().rect()
                    if yeniPos.x() < 10:
                        yeniPos.setX(10)
                        # self.dikey_dondur()
                        # self.anaLay.setDirection(QVBoxLayout.Direction.LeftToRight)
                    elif yeniPos.x() + self.width() > pr.right() - 9:
                        yeniPos.setX(pr.right() - self.width() - 9)
                        # self.dikey_dondur()
                        # self.anaLay.setDirection(QVBoxLayout.Direction.RightToLeft)

                    # if yeniPos.y() < 0:
                    #     yeniPos.setY(0)
                    if yeniPos.y() < 30:
                        yeniPos.setY(30)
                        # self.yatay_dondur()
                        # self.anaLay.setDirection(QVBoxLayout.Direction.TopToBottom)
                    elif yeniPos.y() + self.height() > pr.bottom():
                        yeniPos.setY(pr.bottom() - self.height())
                        # self.yatay_dondur()
                        # self.anaLay.setDirection(QVBoxLayout.Direction.BottomToTop)

                    self.move(yeniPos)

            if self.sagClick:
                genislik = max(self.icerikScroll.minimumWidth(),
                               self.enSagX + event.x() - self.sagDragx)
                yukseklik = max(self.icerikScroll.minimumHeight(),
                                self.enAltY + event.y() - self.sagDragy)
                self.resize(genislik, yukseklik)

                # if self.dikey_mi:
                #     self.baslikEtiket.setFixedHeight(yukseklik)
                #     # self.baslikEtiket.resize(self.baslikEtiket.size().width(), yukseklik)
                # else:
                #     self.baslikEtiket.setFixedWidth(genislik)
                #     # self.baslikEtiket.resize(genislik, self.baslikEtiket.size().height())

                # self.eskiSize = QSize(genislik, yukseklik)

        else:
            if event.buttons() == Qt.MouseButton.LeftButton:
                drag = QDrag(self)
                mime = QMimeData()
                drag.setMimeData(mime)

                pixmap = QPixmap(self.size())
                self.render(pixmap)
                painter = QPainter(pixmap)
                painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_Screen)
                # painter.setOpacity(50)
                # painter.fillRect(pixmap.rect(), QColor(200, 210, 255))
                painter.setPen(QPen(QColor(0, 191, 255), 5, Qt.PenStyle.DotLine))
                painter.drawRect(pixmap.rect())
                painter.end()
                # drag.setHotSpot(QPoint(pixmap.width()/2, pixmap.height()))
                drag.setHotSpot(event.pos() - self.rect().topLeft())
                drag.setPixmap(pixmap)

                drag.exec(Qt.DropAction.MoveAction)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        super(YuzenWidget, self).mouseReleaseEvent(event)

        if not self.cubukta_mi:
            self.sagClick = False
            self.solClick = False
            self.setCursor(Qt.CursorShape.ArrowCursor)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '21/Aug/2016'

from PySide6.QtCore import QUrl
from PySide6.QtWebEngineCore import QWebEngineSettings  # , QWebEngineProfile, QWebEnginePage
from PySide6.QtWidgets import QGraphicsProxyWidget
from PySide6.QtWebEngineWidgets import QWebEngineView
from ..nesneler.base import BaseItem
from .. import shared


########################################################################
class Web(BaseItem):
    Type = shared.WEB_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, html, dosyaAdresi, pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent=None):
        super(Web, self).__init__(pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent)

        # web = QWebEngineView(self)
        # web.setAttribute(Qt.WA_DontShowOnScreen)
        # web.settings().setAttribute(QWebEngineSettings.PluginsEnabled, True)
        # web.settings().setAttribute(QWebEngineSettings.ScreenCaptureEnabled, True)
        # web.load(QUrl.fromLocalFile(dosyaAdresi))

        self.proxy = QGraphicsProxyWidget(self)
        # self.proxy.setFlags(self.ItemIsSelectable |
        #               self.ItemIsMovable |
        #               self.ItemIsFocusable)

        self.wView = QWebEngineView()
        # self.wView.setAttribute(Qt.WA_DontShowOnScreen)
        self.wView.settings().setAttribute(QWebEngineSettings.WebAttribute.PluginsEnabled, False)
        if html:
            self.wView.setHtml(html)
        else:
            self.wView.load(QUrl.fromLocalFile(dosyaAdresi))

        # self.proxy.resize(self.wView.size())
        # self.proxy.resize(self.wView.size())
        # self.proxy.geometryChanged.connect(self.proxy_geometrisi_degisince_doc_size_ayarla)
        # self.proxy.setGeometry()

        self.proxy.setWidget(self.wView)
        # self.setRect(QRectF(self.wView.rect()))
        # self.proxy.resize(QSizeF(self.wView.size()))

        self.wView.resize(self._rect.size().toSize())

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Web.Type

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

import filecmp
import os
import shutil

from PySide6.QtGui import QUndoCommand
from PySide6.QtWidgets import QGraphicsItem

from .nesneler.group import Group
from .nesneler.base import BaseItem
from . import shared


# # ---------------------------------------------------------------------
# def tum_parentitemleri_tara_ve_grup_tipinde_olanlarin_boundingrectini_guncelle(nesne):
#     if nesne.parentItem():
#         if nesne.parentItem().type() == shared.GROUP_ITEM_TYPE:
#             nesne.parentItem().updateBoundingRect()
#         tum_parentitemleri_tara_ve_grup_tipinde_olanlarin_boundingrectini_guncelle(nesne.parentItem())


#######################################################################
class UndoRedoBaglantisiYaziNesnesiDocuna(QUndoCommand):
    # ---------------------------------------------------------------------
    def __init__(self, description, doc, parent=None):
        self.doc = doc
        super(UndoRedoBaglantisiYaziNesnesiDocuna, self).__init__(description, parent)

    # ---------------------------------------------------------------------
    def undo(self) -> None:
        self.doc.undo()

    # ---------------------------------------------------------------------
    def redo(self) -> None:
        self.doc.redo()


########################################################################
class UndoableSayfaAdiDegistir(QUndoCommand):
    """ """

    # TODO: sayfa ve itemin burda referanslarinin tutulmasi problem olur mu acaba,
    # tab kapatılınca veya sayfa silinince silinmiş gozukuyorlar gibi ama detaylı da bakılmadı.

    # ---------------------------------------------------------------------
    def __init__(self, description, sayfa, sayfa_eski_adi, parent=None):
        super(UndoableSayfaAdiDegistir, self).__init__(description, parent)

        self.sayfa = sayfa
        self.sayfa_adi = sayfa.adi
        self.sayfa_eski_adi = sayfa_eski_adi

    # ---------------------------------------------------------------------
    def redo(self):
        # TODO: gerek var mı blocklamaya?? ?
        # self.sayfa.model().blockSignals(True)
        self.sayfa.adi = self.sayfa_adi
        # self.sayfa.setData(self.sayfa_adi, Qt.DisplayRole)
        # self.sayfa.model().blockSignals(False)

    # ---------------------------------------------------------------------
    def undo(self):
        # self.sayfa.model().blockSignals(True)
        self.sayfa.adi = self.sayfa_eski_adi
        # self.sayfa.setData(self.eski_sayfa_adi, Qt.DisplayRole)
        # self.sayfa.model().blockSignals(False)


########################################################################
class UndoableAddItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, scene, item, parent=None):
        super(UndoableAddItem, self).__init__(description, parent)
        self.scene = scene
        self.item = item

    # ---------------------------------------------------------------------
    def redo(self):
        self.scene.parent().act_aramayi_temizle()

        self.scene.addItem(self.item)
        if isinstance(self.item, BaseItem):
            self.item.update_resize_handles()
        self.scene.clearSelection()
        # setSelected iptal edildi. Nesne sayisi cok olan dosyalarin acilis hizi yari yariyadan fazla dusuyor.
        # self.item.setSelected(self.sec)
        # alt satiri burda cagirmiyoruz cunku bazi nesneler ilk once bos olarak ekleniyor, icerikleri sonra ekleniyor.
        # self.scene.unite_with_scene_rect(self.item.mapRectToScene(self.item.boundingRect()))

        # oklar varsa oka bu nesneyi kaydet
        if self.item.oklar_dxdy_nokta:
            for ok, dx_dy_nokta in self.item.oklar_dxdy_nokta.items():
                ok.baglanmis_nesneler[self.item._kim] = dx_dy_nokta[2]

        # silinen ok ise, okun eski bagli oldugu nesnelere okun bilgisini tekrar ekliyoruz
        if self.item.type() == shared.LINE_ITEM_TYPE:
            if self.item.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.item.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.scene._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    if nokta == 1:
                        baglanmis_nesne.ok_ekle(self.item, self.item.mapToScene(self.item._line.p1()),
                                                1)
                    if nokta == 2:
                        baglanmis_nesne.ok_ekle(self.item, self.item.mapToScene(self.item._line.p2()),
                                                2)

    # ---------------------------------------------------------------------
    def undo(self):
        self.scene.parent().act_aramayi_temizle()

        # oklar varsa, oktan bu nesneyi sil, (silinen nesne bir ok nesnesi degil)
        if self.item.oklar_dxdy_nokta:
            for ok in self.item.oklar_dxdy_nokta.keys():
                del ok.baglanmis_nesneler[self.item._kim]

        # silinen ok ise, okun baglanmis oldugu nesnelerden okun bilgisini siliyoruz
        if self.item.type() == shared.LINE_ITEM_TYPE:
            if self.item.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.item.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.scene._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    del baglanmis_nesne.oklar_dxdy_nokta[self.item]

        self.scene.clearSelection()
        self.scene.removeItem(self.item)


########################################################################
class UndoableRemoveItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, scene, item, addToUnGroupedRootItems=False, parent=None):
        super(UndoableRemoveItem, self).__init__(description, parent)
        self.item = item
        self.scene = scene
        self.itemParent = item.parentItem()
        self.addToUnGroupedRootItems = addToUnGroupedRootItems

    # ---------------------------------------------------------------------
    def redo(self):
        self.scene.parent().act_aramayi_temizle()

        self.scene.clearSelection()
        if self.item.type() == shared.VIDEO_ITEM_TYPE:
            self.item.nesne_sahneden_silinmek_uzere()
        self.scene.removeItem(self.item)
        # # ilk silerken, itemlar secili ve selectionQueue icindeler,
        # # ama undo edip sahneye geri aldik ve de her seyi secisiz halde getirip
        # # sonra tekrar redo ile ikinci defa sildigimizde bu sefer az once secisiz hale geldiklerinde
        # # selectionQueue dan silinmeleri sebebi ile, bu ikinci redo da alt satirda tekrar silinmeye calisildiklarindan
        # # hata oluyor. dolayisi ile if ile kontrol ediyoruz.
        # if self.item in self.scene.selectionQueue:
        #     self.scene.selectionQueue.remove(self.item)

        if self.addToUnGroupedRootItems:
            self.scene.unGroupedRootItems.discard(self.item)

        # oklar varsa, oktan bu nesneyi sil, (silinen nesne bir ok nesnesi degil)
        if self.item.oklar_dxdy_nokta:
            for ok in self.item.oklar_dxdy_nokta.keys():
                del ok.baglanmis_nesneler[self.item._kim]

        # silinen ok ise, okun baglanmis oldugu nesnelerden okun bilgisini siliyoruz
        if self.item.type() == shared.LINE_ITEM_TYPE:
            if self.item.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.item.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.scene._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    del baglanmis_nesne.oklar_dxdy_nokta[self.item]

    # # ---------------------------------------------------------------------
    # def append_all_childitems_in_hierarchy_to_selection_queue(self, i):
    #     for c in i.childItems():
    #         if c.childItems():
    #             self.append_all_childitems_in_hierarchy_to_selection_queue(c)
    #         # tempList.append(c)
    #         self.scene.selectionQueue.append(c, simpleAppend=True)

    # ---------------------------------------------------------------------
    def undo(self):
        self.scene.parent().act_aramayi_temizle()

        # self.scene.selectionQueue.append(self.item)
        # # normalde en alt satirdaki select_all_children_recursively halletmesi lazim
        # # ama onu itemlari mouseReleasinde kullaniyoruz super().mouseRelease den sonra
        # # dolayısı ile super() secmiş oluyor ve de selectionQueue ya eklenmis oluyorlar
        # # yine dolayisi ile biz select_all_children_recursively de secsek de, tekrar eklemiyoruz
        # # selectionQueue ya. o yuzden burda recursive eklemek durumundayiz .
        # self.append_all_childitems_in_hierarchy_to_selection_queue(self.item)

        self.scene.addItem(self.item)
        if self.itemParent:
            self.item.setParentItem(self.itemParent)
            if self.itemParent.type() == shared.GROUP_ITEM_TYPE:
                self.itemParent.parentedWithParentOperation.append(self.item)
        if self.addToUnGroupedRootItems:
            self.scene.unGroupedRootItems.add(self.item)
        # self.scene.select_all_children_recursively(self.item, cosmeticSelect=False, topmostParent=True)

        # oklar varsa oka bu nesneyi kaydet
        if self.item.oklar_dxdy_nokta:
            for ok, dx_dy_nokta in self.item.oklar_dxdy_nokta.items():
                ok.baglanmis_nesneler[self.item._kim] = dx_dy_nokta[2]

        # silinen ok ise, okun eski bagli oldugu nesnelere okun bilgisini tekrar ekliyoruz
        if self.item.type() == shared.LINE_ITEM_TYPE:
            if self.item.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.item.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.scene._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    if nokta == 1:
                        baglanmis_nesne.ok_ekle(self.item, self.item.mapToScene(self.item._line.p1()),
                                                1)
                    if nokta == 2:
                        baglanmis_nesne.ok_ekle(self.item, self.item.mapToScene(self.item._line.p2()),
                                                2)


########################################################################
class UndoableMove(QUndoCommand):
    # TODO:  bazen moved 0 item diyor ozellikle kaydedilmis dosya yukleyip icerigini hareket ettirince.
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, movedItem, eskiPosition, parent=None):
        """ """
        super(UndoableMove, self).__init__(description, parent)

        self.movedItem = movedItem
        self.eskiPosition = eskiPosition
        self.yeniPosition = self.movedItem.pos()

    # ---------------------------------------------------------------------
    def undo(self):
        # self.movedItem.moveBy(-self.dx, -self.dy)
        self.movedItem.setPos(self.eskiPosition)
        self.movedItem.ok_guncelle()

        # ok bagli iken nesne olarak tasinirsa, bagli oldugu nesnelere bagliligi devam ettigi halde baglanti noktalarina
        # olan bagi gevsetiliyor (offset)
        if self.movedItem.type() == shared.LINE_ITEM_TYPE:
            if self.movedItem.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.movedItem.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.movedItem.scene()._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    if nokta == 1:
                        baglanmis_nesne.ok_ekle(self.movedItem, self.movedItem.mapToScene(self.movedItem._line.p1()),
                                                1)
                    if nokta == 2:
                        baglanmis_nesne.ok_ekle(self.movedItem, self.movedItem.mapToScene(self.movedItem._line.p2()),
                                                2)

    # ---------------------------------------------------------------------
    def redo(self):
        # self.movedItem.moveBy(self.dx, self.dy)
        self.movedItem.setPos(self.yeniPosition)
        self.movedItem.ok_guncelle()

        # ok bagli iken nesne olarak tasinirsa, bagli oldugu nesnelere bagliligi devam ettigi halde baglanti noktalarina
        # olan bagi gevsetiliyor (offset)
        if self.movedItem.type() == shared.LINE_ITEM_TYPE:
            if self.movedItem.baglanmis_nesneler:
                for baglanmis_nesne_kimlik, nokta in self.movedItem.baglanmis_nesneler.items():  # dict items()
                    baglanmis_nesne = self.movedItem.scene()._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
                    if nokta == 1:
                        baglanmis_nesne.ok_ekle(self.movedItem, self.movedItem.mapToScene(self.movedItem._line.p1()),
                                                1)
                    if nokta == 2:
                        baglanmis_nesne.ok_ekle(self.movedItem, self.movedItem.mapToScene(self.movedItem._line.p2()),
                                                2)


########################################################################
class UndoableMovePathPoint(QUndoCommand):

    # ---------------------------------------------------------------------
    def __init__(self, description, item, movedPointIndex, eskiPosTuple, yeniPosTuple, parent=None):
        """ """
        super(UndoableMovePathPoint, self).__init__(description, parent)

        self.item = item
        self.movedPointIndex = movedPointIndex
        self.eskiPosTuple = eskiPosTuple
        self.yeniPosTuple = yeniPosTuple

    # ---------------------------------------------------------------------
    def undo(self):
        self.item._path.setElementPositionAt(self.movedPointIndex, self.eskiPosTuple[0], self.eskiPosTuple[1])
        self.item.update()

    # ---------------------------------------------------------------------
    def redo(self):
        self.item._path.setElementPositionAt(self.movedPointIndex, self.yeniPosTuple[0], self.yeniPosTuple[1])
        self.item.update()


########################################################################
class UndoableDeletePathPoint(QUndoCommand):

    # ---------------------------------------------------------------------
    def __init__(self, description, item, eskiPath, yeniPath, parent=None):
        """ """
        super(UndoableDeletePathPoint, self).__init__(description, parent)

        self.item = item
        self.eskiPath = eskiPath
        self.yeniPath = yeniPath

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPath(self.eskiPath)
        # self.item.update()

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPath(self.yeniPath)
        # self.item.update()


########################################################################
class UndoableGroup(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, items, scene, parent=None):
        super(UndoableGroup, self).__init__(description, parent)

        self.group = Group()
        self.items = items
        self.scene = scene

    # ---------------------------------------------------------------------
    def redo(self):
        self.scene.addItem(self.group)
        self.scene.parent().increase_zvalue(self.group)
        for item in self.items:
            if item.parentItem():
                if item.parentItem() in self.items:  # o zaman bunu gruplamaya gerek yok cunku parenti gruplanacak zaten.
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, True)
                    # muhtemelen parentta da sorun var
                    # continue
                else:
                    self.scene.parent().act_unparent_items(item)
                    self.group.addSingleItemToGroup(item)
                    # TODO: if ile kontrol etsek once daha mi hizli olur, time complexity ye bakmak lazim.
                    # gruplandigi icin referansi parentta olacak ayrica tutmaya gerek yok
                    # burda da ihtiyac var discarda cunku yukarda unparent edilince ekleniyor unGroupedRootItems a.
                    self.scene.unGroupedRootItems.discard(item)
            else:
                if item.childItems():  # sadece parent secilmis iken alt nesneleri secilemez hale getiriyoruz.
                    for c in item.childItems():
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, True)
                self.group.addSingleItemToGroup(item)
                # TODO: if ile kontrol etsek once daha mi hizli olur, time complexity ye bakmak lazim.
                # gruplandigi icin referansi parentta olacak ayrica tutmaya gerek yok
                self.scene.unGroupedRootItems.discard(item)
        self.group.updateBoundingRect()
        self.group.setSelected(True)

    # ---------------------------------------------------------------------
    def undo(self):
        self.group.destroyGroup()
        self.scene.removeItem(self.group)


########################################################################
class UndoableUnGroup(QUndoCommand):
    """ """

    def __init__(self, description, group, parent=None):
        super(UndoableUnGroup, self).__init__(description, parent)
        self.group = group
        self.scene = group.scene()
        self.itemsList = group.allFirstLevelGroupChildren
        self.itemsListWithPos = [[item, item.pos()] for item in group.allFirstLevelGroupChildren]
        # self.itemsDictWithPos = {item: item.pos() for item in group.allFirstLevelGroupChildren}
        # import sys
        # print(sys.getsizeof(self.itemsListWithPos))
        # print(sys.getsizeof(self.itemsDictWithPos))

    # ---------------------------------------------------------------------
    def redo(self):
        self.group.destroyGroup()
        self.scene.removeItem(self.group)
        # self.group.scene().removeItem(self.group)
        # oklar varsa, oktan bu nesneyi sil, (silinen nesne bir ok nesnesi degil)
        if self.group.oklar_dxdy_nokta:
            for ok in self.group.oklar_dxdy_nokta.keys():
                del ok.baglanmis_nesneler[self.group._kim]

    # ---------------------------------------------------------------------
    def undo(self):

        # group = Group()
        # # group = QGraphicsItemGroup(None)
        # group.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
        #                |QGraphicsItem.GraphicsItemFlag.ItemIsMovable
        #                # | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable
        #                )

        self.scene.addItem(self.group)
        # #### self.scene.parent().increase_zvalue(self.group)
        for item, itemEskiPos in self.itemsListWithPos:
            if item.parentItem():
                if item.parentItem() in self.itemsList:  # o zaman bunu gruplamaya gerek yok cunku parenti gruplanacak zaten.
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, False)
                    item.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, True)
                    # muhtemelen parentta da sorun var
                    # continue
                else:
                    # self.scene.parent().act_unparent_items(item)
                    self.group.addSingleItemToGroup(item)
                    # TODO: if ile kontrol etsek once daha mi hizli olur, time complexity ye bakmak lazim.
                    # gruplandigi icin referansi parentta olacak ayrica tutmaya gerek yok
                    # burda da ihtiyac var discarda cunku yukarda unparent edilince ekleniyor unGroupedRootItems a.
                    self.scene.unGroupedRootItems.discard(item)
            else:
                if item.childItems():  # sadece parent secilmis iken alt nesneleri secilemez hale getiriyoruz.
                    for c in item.childItems():
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsFocusable, False)
                        c.setFlag(QGraphicsItem.GraphicsItemFlag.ItemStacksBehindParent, True)
                self.group.addSingleItemToGroup(item)
                # TODO: if ile kontrol etsek once daha mi hizli olur, time complexity ye bakmak lazim.
                # gruplandigi icin referansi parentta olacak ayrica tutmaya gerek yok
                self.scene.unGroupedRootItems.discard(item)
            item.setPos(itemEskiPos)
        # #### self.group.updateBoundingRect()
        # oklar varsa oka bu nesneyi kaydet
        if self.group.oklar_dxdy_nokta:
            for ok, dx_dy_nokta in self.group.oklar_dxdy_nokta.items():
                ok.baglanmis_nesneler[self.group._kim] = dx_dy_nokta[2]

        self.group.setSelected(True)


########################################################################
class UndoableParent(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, parentItem, yeniPos, parent=None):
        super(UndoableParent, self).__init__(description, parent)
        self.item = item
        self.parentItem = parentItem
        self.parentRot = 0
        while parentItem:
            # cunku itemRot mesela 40 derece ise, yeni parentRot 10 derece ise, item parent edilince 40-10 = 30 oluyor.
            # dolayısı ile ne kadar parent icinde ise hepsinin rotation toplami asil sceneRotation unu veriyor.
            self.parentRot += parentItem.rotation()
            parentItem = parentItem.parentItem()
        self.eskiParentItem = item.parentItem()
        self.yeniPos = yeniPos
        self.eskiPos = item.pos()
        self.eskiRot = item.rotation()

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setParentItem(self.parentItem)
        self.item.setPos(self.yeniPos)
        self.item.setRotation(self.item.rotation() - self.parentRot)
        # if self.parentItem:
        if self.parentItem.type() == shared.GROUP_ITEM_TYPE:
            self.parentItem.parentedWithParentOperation.append(self.item)
        self.item.scene().unGroupedRootItems.discard(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        if self.eskiParentItem:
            self.item.setParentItem(self.eskiParentItem)
            # eger self.eskiParentItem grupsa, parentedWithParentOperation dan silmeye gerek yok cunku
            # onu grupta hallediyoruz.
        else:
            self.item.scene().unGroupedRootItems.add(self.item)
            self.item.setParentItem(None)

        self.item.setPos(self.eskiPos)
        self.item.setRotation(self.eskiRot)


########################################################################
class UndoableUnParent(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, yeniParentItem, yeniPos, parent=None):
        super(UndoableUnParent, self).__init__(description, parent)
        self.item = item
        self.yeniParentItem = yeniParentItem

        self.yeniPos = yeniPos
        # self.eskiPos = item.scenePos()
        self.eskiPos = item.pos()

        self.eskiParentItem = item.parentItem()
        eskiParentItem = self.eskiParentItem
        self.eskiParentRot = 0
        while eskiParentItem:
            self.eskiParentRot += eskiParentItem.rotation()
            eskiParentItem = eskiParentItem.parentItem()

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.scene().unGroupedRootItems.add(self.item)
        self.item.setPos(self.yeniPos)
        self.item.setRotation(self.item.rotation() + self.eskiParentRot)
        self.item.setParentItem(self.yeniParentItem)
        # eger self.eskiParentItem grupsa, parentedWithParentOperation dan silmeye gerek yok cunku
        # onu grupta hallediyoruz.

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setParentItem(self.eskiParentItem)
        self.item.setPos(self.eskiPos)
        self.item.setRotation(self.item.rotation() - self.eskiParentRot)
        # if self.eskiParentItem:
        if self.eskiParentItem.type() == shared.GROUP_ITEM_TYPE:
            self.eskiParentItem.parentedWithParentOperation.append(self.item)
        self.item.scene().unGroupedRootItems.discard(self.item)


########################################################################
class UndoableSetFont(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, font, parent=None):
        super(UndoableSetFont, self).__init__(description, parent)
        self.item = item
        self.font = font
        self.eskiFont = item.font()

    # ---------------------------------------------------------------------
    def id(self):
        return 1

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.font = enSonUndo.font
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setFont(self.font)
        self.item.scene().parent().change_font_combobox_value(self.font)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setFont(self.eskiFont)
        self.item.scene().parent().change_font_combobox_value(self.eskiFont)


########################################################################
class UndoableSetFontSizeF(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, fontPointSizeF, parent=None):
        super(UndoableSetFontSizeF, self).__init__(description, parent)
        self.item = item
        self.fontPointSizeF = fontPointSizeF
        self.eskiFontPointSizeF = item.font().pointSizeF()

    # ---------------------------------------------------------------------
    def id(self):
        return 2

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        # TODO: for now this is useless because it is in a macro. and other ones are too,
        # which are called from a macro.
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.fontPointSizeF = enSonUndo.fontPointSizeF
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setFontPointSizeF(self.fontPointSizeF)
        self.item.scene().parent().change_font_point_sizef_spinbox_value(self.fontPointSizeF)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setFontPointSizeF(self.eskiFontPointSizeF)
        self.item.scene().parent().change_font_point_sizef_spinbox_value(self.eskiFontPointSizeF)


########################################################################
class UndoableSetCharacterFormat(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, bicim, parent=None):
        super(UndoableSetCharacterFormat, self).__init__(description, parent)
        self.item = item
        self.bicim = bicim
        self.eski_bicim = item.ver_karakter_bicimi()

    # # ---------------------------------------------------------------------
    # def id(self):
    #     return 3
    #
    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #     self.bicim = enSonUndo.bicim
    #     return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.kur_karakter_bicimi(self.bicim)
        self.item.scene().parent().karakter_bicimi_degisti(self.bicim)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.kur_karakter_bicimi(self.eski_bicim)
        self.item.scene().parent().karakter_bicimi_degisti(self.eski_bicim)


########################################################################
class UndoableSetTextAlignment(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, hiza, parent=None):
        super(UndoableSetTextAlignment, self).__init__(description, parent)
        self.item = item
        self.hiza = hiza
        self.eski_hiza = item.ver_yazi_hizasi()

    # # ---------------------------------------------------------------------
    # def id(self):
    #     return 4
    #
    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #     self.hiza = enSonUndo.hiza
    #     return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.kur_yazi_hizasi(self.hiza)
        self.item.scene().parent().yazi_hizalama_degisti(self.hiza)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.kur_yazi_hizasi(self.eski_hiza)
        self.item.scene().parent().yazi_hizalama_degisti(self.eski_hiza)


########################################################################
class UndoableResizeBaseItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, yeniRect, eskiRect, eskiPos, parent=None):
        super(UndoableResizeBaseItem, self).__init__(description, parent)
        self.item = item
        self.yeniPos = self.item.pos()
        self.yeniRect = yeniRect
        self.eskiPos = eskiPos
        self.eskiRect = eskiRect

    # ---------------------------------------------------------------------
    # def id(self):
    #     return 5
    #
    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #
    #     self.yeniRect = enSonUndo.yeniRect
    #     return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPos(self.yeniPos)
        self.item.setRect(self.yeniRect)
        # if not self.item.type() == shared.TEXT_ITEM_TYPE:
        #     self.item.repositionChildItems(self.eskiPos - self.yeniPos)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)
        if self.item.type() == shared.IMAGE_ITEM_TYPE:
            self.item.reload_image_after_scale()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPos(self.eskiPos)
        self.item.setRect(self.eskiRect)
        # if not self.item.type() == shared.TEXT_ITEM_TYPE:
        #     self.item.repositionChildItems(self.yeniPos - self.eskiPos)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)
        if self.item.type() == shared.IMAGE_ITEM_TYPE:
            self.item.reload_image_after_scale()


########################################################################
class UndoableResizeLineItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, yeniLine, eskiLine, eskiPos, degisenNokta=None, parent=None):
        super(UndoableResizeLineItem, self).__init__(description, parent)
        self.cizgiNesnesi = item
        self.scene = item.scene()
        self.yeniPos = item.pos()
        self.yeniLine = yeniLine
        self.eskiPos = eskiPos
        self.eskiLine = eskiLine
        self.degisenNokta = degisenNokta

        self.p1_in_baglandigi_eski_nesne = None
        self.p2_in_baglandigi_eski_nesne = None
        for baglanmis_nesne_kimlik, nokta in self.cizgiNesnesi.baglanmis_nesneler.items():  # dict.items()
            baglanmis_nesne = self.scene._kimlik_nesne_sozluk[baglanmis_nesne_kimlik]
            if 1 == nokta:
                self.p1_in_baglandigi_eski_nesne = baglanmis_nesne
            if 2 == nokta:
                self.p2_in_baglandigi_eski_nesne = baglanmis_nesne

    # ---------------------------------------------------------------------
    # def id(self):
    #     return 6
    #
    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.cizgiNesnesi is not enSonUndo.cizgiNesnesi:
    #         return False
    #
    #     self.yeniRect = enSonUndo.yeniRect
    #     return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.cizgiNesnesi.setPos(self.yeniPos)
        self.cizgiNesnesi.setLine(self.yeniLine)
        self.cizgiNesnesi.repositionChildItems(self.eskiPos - self.yeniPos)
        self.cizgiNesnesi.update_resize_handles()
        # self.scene.parent().change_transform_box_values(self.cizgiNesnesi)

        if self.degisenNokta == 1:
            p1_scene_pos = self.cizgiNesnesi.mapToScene(self.yeniLine.p1())
            ustuneOkCizilenItemList_p1 = self.scene.items(p1_scene_pos,
                                                          deviceTransform=self.scene.views()[0].transform())
            # bazen veya (eskiden?) cizilen ok da listelenebiliyor(du).
            if self.cizgiNesnesi in ustuneOkCizilenItemList_p1:
                ustuneOkCizilenItemList_p1.remove(self.cizgiNesnesi)
            if ustuneOkCizilenItemList_p1:
                p1_in_baglanacagi_yeni_nesne = ustuneOkCizilenItemList_p1[0]
                enustGrup = p1_in_baglanacagi_yeni_nesne.varsaEnUsttekiGrubuGetir()
                if enustGrup:
                    p1_in_baglanacagi_yeni_nesne = enustGrup

                if self.p1_in_baglandigi_eski_nesne:  # noktayi bosluktan yeni nesnenin ustune tasiyor da olabiliriz 
                    if not p1_in_baglanacagi_yeni_nesne._kim == self.p1_in_baglandigi_eski_nesne._kim:
                        self.p1_in_baglandigi_eski_nesne.ok_sil(self.cizgiNesnesi)
                p1_in_baglanacagi_yeni_nesne.ok_ekle(self.cizgiNesnesi, p1_scene_pos, 1)
            else:
                # eski nesne uzerinden bosluga tasidik noktayi ve hala eski nesneye bagli, pozisyon ayarlamasi yapiliyor
                # bir benzeri UndoableMove() da var. Ok nesne olarak tasinirsa ve bagli oldugu nesneler varsa
                # bagli oldugu noktalarin pozisyon ayarlamasi yapilior (offset)
                if self.p1_in_baglandigi_eski_nesne:
                    self.p1_in_baglandigi_eski_nesne.ok_ekle(self.cizgiNesnesi, p1_scene_pos, 1)

        elif self.degisenNokta == 2:
            p2_scene_pos = self.cizgiNesnesi.mapToScene(self.yeniLine.p2())
            ustuneOkCizilenItemList_p2 = self.scene.items(p2_scene_pos,
                                                          deviceTransform=self.scene.views()[0].transform())
            # bazen veya (eskiden?) cizilen ok da listelenebiliyor(du).
            if self.cizgiNesnesi in ustuneOkCizilenItemList_p2:
                ustuneOkCizilenItemList_p2.remove(self.cizgiNesnesi)
            if ustuneOkCizilenItemList_p2:
                p2_in_baglanacagi_yeni_nesne = ustuneOkCizilenItemList_p2[0]
                enustGrup = p2_in_baglanacagi_yeni_nesne.varsaEnUsttekiGrubuGetir()
                if enustGrup:
                    p2_in_baglanacagi_yeni_nesne = enustGrup

                if self.p2_in_baglandigi_eski_nesne:  # noktayi bosluktan yeni nesnenin ustune tasiyor da olabiliriz
                    if not p2_in_baglanacagi_yeni_nesne._kim == self.p2_in_baglandigi_eski_nesne._kim:
                        self.p2_in_baglandigi_eski_nesne.ok_sil(self.cizgiNesnesi)
                p2_in_baglanacagi_yeni_nesne.ok_ekle(self.cizgiNesnesi, p2_scene_pos, 2)
            else:
                # eski nesne uzerinden bosluga tasidik noktayi ve hala eski nesneye bagli, pozisyon ayarlamasi yapiliyor
                # bir benzeri UndoableMove() da var. Ok nesne olarak tasinirsa ve bagli oldugu nesneler varsa
                # bagli oldugu noktalarin pozisyon ayarlamasi yapilior (offset)
                if self.p2_in_baglandigi_eski_nesne:
                    self.p2_in_baglandigi_eski_nesne.ok_ekle(self.cizgiNesnesi, p2_scene_pos, 2)
        # else:
        #     print(self.degisenNokta)

    # ---------------------------------------------------------------------
    def undo(self):
        self.cizgiNesnesi.setPos(self.eskiPos)
        self.cizgiNesnesi.setLine(self.eskiLine)
        self.cizgiNesnesi.repositionChildItems(self.yeniPos - self.eskiPos)
        self.cizgiNesnesi.update_resize_handles()
        # self.scene.parent().change_transform_box_values(self.cizgiNesnesi)

        if self.degisenNokta == 1:
            p1_scene_pos = self.cizgiNesnesi.mapToScene(self.eskiLine.p1())
            ustuneOkCizilenItemList_p1 = self.scene.items(p1_scene_pos,
                                                          deviceTransform=self.scene.views()[0].transform())
            # bazen veya (eskiden?) cizilen ok da listelenebiliyor(du).
            if self.cizgiNesnesi in ustuneOkCizilenItemList_p1:
                ustuneOkCizilenItemList_p1.remove(self.cizgiNesnesi)
            if ustuneOkCizilenItemList_p1:
                p1_in_baglanacagi_yeni_nesne = ustuneOkCizilenItemList_p1[0]
                enustGrup = p1_in_baglanacagi_yeni_nesne.varsaEnUsttekiGrubuGetir()
                if enustGrup:
                    p1_in_baglanacagi_yeni_nesne = enustGrup

                if self.p1_in_baglandigi_eski_nesne:  # noktayi bosluktan yeni nesnenin ustune tasiyor da olabiliriz
                    if not p1_in_baglanacagi_yeni_nesne._kim == self.p1_in_baglandigi_eski_nesne._kim:
                        self.p1_in_baglandigi_eski_nesne.ok_sil(self.cizgiNesnesi)
                p1_in_baglanacagi_yeni_nesne.ok_ekle(self.cizgiNesnesi, p1_scene_pos, 1)
            else:
                # eski nesne uzerinden bosluga tasidik noktayi ve hala eski nesneye bagli, pozisyon ayarlamasi yapiliyor
                # bir benzeri UndoableMove() da var. Ok nesne olarak tasinirsa ve bagli oldugu nesneler varsa
                # bagli oldugu noktalarin pozisyon ayarlamasi yapilior (offset)
                if self.p1_in_baglandigi_eski_nesne:
                    self.p1_in_baglandigi_eski_nesne.ok_ekle(self.cizgiNesnesi, p1_scene_pos, 1)

        elif self.degisenNokta == 2:
            p2_scene_pos = self.cizgiNesnesi.mapToScene(self.eskiLine.p2())
            ustuneOkCizilenItemList_p2 = self.scene.items(p2_scene_pos,
                                                          deviceTransform=self.scene.views()[0].transform())
            # bazen veya (eskiden?) cizilen ok da listelenebiliyor(du).
            if self.cizgiNesnesi in ustuneOkCizilenItemList_p2:
                ustuneOkCizilenItemList_p2.remove(self.cizgiNesnesi)
            if ustuneOkCizilenItemList_p2:
                p2_in_baglanacagi_yeni_nesne = ustuneOkCizilenItemList_p2[0]
                enustGrup = p2_in_baglanacagi_yeni_nesne.varsaEnUsttekiGrubuGetir()
                if enustGrup:
                    p2_in_baglanacagi_yeni_nesne = enustGrup

                if self.p2_in_baglandigi_eski_nesne:  # noktayi bosluktan yeni nesnenin ustune tasiyor da olabiliriz
                    if not p2_in_baglanacagi_yeni_nesne._kim == self.p2_in_baglandigi_eski_nesne._kim:
                        self.p2_in_baglandigi_eski_nesne.ok_sil(self.cizgiNesnesi)
                p2_in_baglanacagi_yeni_nesne.ok_ekle(self.cizgiNesnesi, p2_scene_pos, 2)
            else:
                # eski nesne uzerinden bosluga tasidik noktayi ve hala eski nesneye bagli, pozisyon ayarlamasi yapiliyor
                # bir benzeri UndoableMove() da var. Ok nesne olarak tasinirsa ve bagli oldugu nesneler varsa
                # bagli oldugu noktalarin pozisyon ayarlamasi yapilior (offset)
                if self.p2_in_baglandigi_eski_nesne:
                    self.p2_in_baglandigi_eski_nesne.ok_ekle(self.cizgiNesnesi, p2_scene_pos, 2)


########################################################################
class UndoableResizeGroupItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, eskiRect, yeniRect, eskiPos, yeniPos, scaleFactorX, scaleFactorY, parent=None):
        super(UndoableResizeGroupItem, self).__init__(description, parent)
        self.item = item
        # self.eskiRect = item._rect
        self.eskiRect = eskiRect
        self.yeniRect = yeniRect
        self.scaleFactorX = scaleFactorX
        self.scaleFactorY = scaleFactorY

        self.eskiPos = eskiPos
        self.yeniPos = yeniPos

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPos(self.yeniPos)
        self.yeniRect.moveTo(0, 0)
        self.item.setRect(self.yeniRect)
        # Dikkat: Demiyoruz ki; burda gerek yok zaten mouseMoveda bu islem hallediliyor
        # geri-al ileri-al yapılırsa ihtiyac olacak
        shared._scaleChildItemsByResizing(self.item, self.scaleFactorX, self.scaleFactorY, True)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPos(self.eskiPos)
        self.item.setRect(self.eskiRect)
        shared._scaleChildItemsByResizing(self.item, 1 / self.scaleFactorX, 1 / self.scaleFactorY, True)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableResizePathItem(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, scaledPath, yeniPos, parent=None):
        super(UndoableResizePathItem, self).__init__(description, parent)
        self.item = item
        self.eskiPath = item.path()
        self.eskiPos = item.pos()
        self.yeniPath = scaledPath
        self.yeniPos = yeniPos

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPath(self.yeniPath)
        self.item.setPos(self.yeniPos)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPath(self.eskiPath)
        self.item.setPos(self.eskiPos)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableScaleTextItemByResizing(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, scaleFactor, fontPointSizeF, parent=None):
        super(UndoableScaleTextItemByResizing, self).__init__(description, parent)
        self.item = item
        self.scaleFactor = scaleFactor
        self.fontPointSizeF = fontPointSizeF
        self.eskiFontPointSizeF = self.item.fontPointSizeF()

    # ---------------------------------------------------------------------
    def id(self):
        return 7

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False

        self.scaleFactor *= enSonUndo.scaleFactor  # dikkat *=
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setFontPointSizeF(self.fontPointSizeF)
        self.item.scene().parent().change_font_point_sizef_spinbox_value(self.fontPointSizeF)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setFontPointSizeF(self.eskiFontPointSizeF)
        self.item.scene().parent().change_font_point_sizef_spinbox_value(self.eskiFontPointSizeF)
        shared._scaleChildItemsByResizing(self.item, 1 / self.scaleFactor)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableScaleGroupItemByResizing(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, yeniRect, scaleFactor, yeniPos, parent=None):
        super(UndoableScaleGroupItemByResizing, self).__init__(description, parent)
        self.item = item
        self.eskiRect = item._rect
        self.yeniRect = yeniRect
        self.scaleFactor = scaleFactor
        self.yeniPos = yeniPos
        self.eskiPos = item.pos()

    # ---------------------------------------------------------------------
    def id(self):
        return 8

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False

        self.yeniRect = enSonUndo.yeniRect
        self.yeniPos = enSonUndo.yeniPos
        self.scaleFactor *= enSonUndo.scaleFactor  # dikkat *=
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setRect(self.yeniRect)
        self.item.setPos(self.yeniPos)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setRect(self.eskiRect)
        self.item.setPos(self.eskiPos)
        shared._scaleChildItemsByResizing(self.item, 1 / self.scaleFactor)
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableScaleBaseItemByResizing(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, yeniRect, scaleFactor, yeniPos, parent=None):
        super(UndoableScaleBaseItemByResizing, self).__init__(description, parent)
        self.item = item
        self.eskiRect = item.rect()
        self.yeniRect = yeniRect
        self.scaleFactor = scaleFactor
        self.eskiPos = item.pos()
        self.yeniPos = yeniPos

    # ---------------------------------------------------------------------
    def id(self):
        return 9

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False

        self.yeniRect = enSonUndo.yeniRect
        self.yeniPos = enSonUndo.yeniPos
        self.scaleFactor *= enSonUndo.scaleFactor  # dikkat *=
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setRect(self.yeniRect)
        self.item.setPos(self.yeniPos)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)
        self.item.update_painter_text_rect()
        if self.item.type() == shared.IMAGE_ITEM_TYPE:
            self.item.reload_image_after_scale()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setRect(self.eskiRect)
        self.item.setPos(self.eskiPos)
        shared._scaleChildItemsByResizing(self.item, 1 / self.scaleFactor)
        self.item.update_resize_handles()
        self.item.scene().parent().change_transform_box_values(self.item)
        self.item.update_painter_text_rect()
        if self.item.type() == shared.IMAGE_ITEM_TYPE:
            self.item.reload_image_after_scale()


########################################################################
class UndoableScalePathItemByScalingPath(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, path, scaleFactor, parent=None):
        super(UndoableScalePathItemByScalingPath, self).__init__(description, parent)
        self.item = item
        self.path = path
        self.eskiPath = item.path()
        self.scaleFactor = scaleFactor

    # ---------------------------------------------------------------------
    def id(self):
        return 10

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.path = enSonUndo.path
        self.scaleFactor *= enSonUndo.scaleFactor  # dikkat *=
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPath(self.path)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPath(self.eskiPath)
        shared._scaleChildItemsByResizing(self.item, 1 / self.scaleFactor)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableScaleLineItemByScalingLine(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, line, scaleFactor, parent=None):
        super(UndoableScaleLineItemByScalingLine, self).__init__(description, parent)
        self.item = item
        self.eskiLine = item.line()
        self.line = line
        self.scaleFactor = scaleFactor

    # ---------------------------------------------------------------------
    def id(self):
        return 11

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.line = enSonUndo.line
        self.scaleFactor *= enSonUndo.scaleFactor  # dikkat *=
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setLine(self.line)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setLine(self.eskiLine)
        shared._scaleChildItemsByResizing(self.item, self.scaleFactor)
        self.item.update_painter_text_rect()
        self.item.scene().parent().change_transform_box_values(self.item)


########################################################################
class UndoableRotate(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, rotation, parent=None):
        super(UndoableRotate, self).__init__(description, parent)
        self.item = item
        self.rotation = rotation % 360
        self.eskiRotation = item.rotation() % 360

    # ---------------------------------------------------------------------
    def id(self):
        return 12

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.rotation = enSonUndo.rotation
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setRotation(self.rotation)
        self.item.scene().parent().itemRotationSBox_tbar.setValue(self.rotation)
        self.item.scene().parent().itemRotationSBox_nesnedw.setValue(self.rotation)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setRotation(self.eskiRotation)
        self.item.scene().parent().itemRotationSBox_tbar.setValue(self.eskiRotation)
        self.item.scene().parent().itemRotationSBox_nesnedw.setValue(self.eskiRotation)


########################################################################
class UndoableRotateWithOffset(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, rotation, parent=None):
        super(UndoableRotateWithOffset, self).__init__(description, parent)
        self.item = item
        self.rotation = rotation % 360
        self.eskiRotation = item.rotation() % 360

    # ---------------------------------------------------------------------
    def id(self):
        return 13

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.rotation = enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.rotateWithOffset(self.rotation)
        self.item.scene().parent().itemRotationSBox_tbar.setValue(self.rotation)
        self.item.scene().parent().itemRotationSBox_nesnedw.setValue(self.rotation)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.rotateWithOffset(self.eskiRotation)
        self.item.scene().parent().itemRotationSBox_tbar.setValue(self.eskiRotation)
        self.item.scene().parent().itemRotationSBox_nesnedw.setValue(self.eskiRotation)


########################################################################
class UndoableSetZValue(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, zValue, parent=None):
        super(UndoableSetZValue, self).__init__(description, parent)
        self.item = item
        self.zValue = zValue
        self.eskiZValue = item.zValue()

    # ---------------------------------------------------------------------
    def id(self):
        return 14

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.zValue is not enSonUndo.zValue:
            return False
        self.zValue = enSonUndo.zValue
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setZValue(self.zValue)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setZValue(self.eskiZValue)


########################################################################
class UndoableSetLineCapStyle(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, capStyle, parent=None):
        super(UndoableSetLineCapStyle, self).__init__(description, parent)
        self.item = item
        self.capStyle = capStyle
        self.eskiCapStyle = item._pen.capStyle()

    # ---------------------------------------------------------------------
    def id(self):
        return 15

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.capStyle = enSonUndo.capStyle
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item._pen.setCapStyle(self.capStyle)
        self.item.update()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item._pen.setCapStyle(self.eskiCapStyle)
        self.item.update()


########################################################################
class UndoableSetLineJoinStyle(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, joinStyle, parent=None):
        super(UndoableSetLineJoinStyle, self).__init__(description, parent)
        self.item = item
        self.joinStyle = joinStyle
        self.eskiJoinStyle = item._pen.joinStyle()

    # ---------------------------------------------------------------------
    def id(self):
        return 16

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.joinStyle = enSonUndo.joinStyle
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item._pen.setJoinStyle(self.joinStyle)
        self.item.update()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item._pen.setJoinStyle(self.eskiJoinStyle)
        self.item.update()


########################################################################
class UndoableSetLineStyle(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, lineStyle, parent=None):
        super(UndoableSetLineStyle, self).__init__(description, parent)
        self.item = item
        self.lineStyle = lineStyle
        self.eskiLineStyle = item._pen.style()

    # ---------------------------------------------------------------------
    def id(self):
        return 17

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.lineStyle = enSonUndo.lineStyle
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item._pen.setStyle(self.lineStyle)
        self.item.update()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item._pen.setStyle(self.eskiLineStyle)
        self.item.update()


########################################################################
class UndoableSetLineWidthF(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, widthF, parent=None):
        super(UndoableSetLineWidthF, self).__init__(description, parent)
        self.item = item
        self.widthF = widthF
        self.eskiWidthF = item._pen.widthF()

    # ---------------------------------------------------------------------
    def id(self):
        return 18

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.widthF = enSonUndo.widthF
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setCizgiKalinligi(self.widthF)
        self.item.scene().parent().change_line_widthF(self.widthF)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setCizgiKalinligi(self.eskiWidthF)
        self.item.scene().parent().change_line_widthF(self.eskiWidthF)


########################################################################
class UndoableSetLineColor(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, renkSecicidenMi, parent=None):
        super(UndoableSetLineColor, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.cizgiRengi
        self.renkSecicidenMi = renkSecicidenMi

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setCizgiRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.color, nesne_arkaplan_ikonu_guncelle=True,
                                                                  renkSecicidenMi=self.renkSecicidenMi)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setCizgiRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.eskiColor, nesne_arkaplan_ikonu_guncelle=True,
                                                                  renkSecicidenMi=self.renkSecicidenMi)


########################################################################
class UndoableSetTextColor(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, renkSecicidenMi, parent=None):
        super(UndoableSetTextColor, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.yaziRengi
        self.renkSecicidenMi = renkSecicidenMi
        # if item.type() == shared.TEXT_ITEM_TYPE:
        #     self.eskiColor = item.defaultTextColor()
        #     # self.setText("change text color")
        # else:
        #     self.eskiColor = item._pen.color()

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setYaziRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.color, nesne_arkaplan_ikonu_guncelle=True,
                                                                 renkSecicidenMi=self.renkSecicidenMi)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setYaziRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.eskiColor, nesne_arkaplan_ikonu_guncelle=True,
                                                                 renkSecicidenMi=self.renkSecicidenMi)


########################################################################
class UndoableSetLineColorAlpha(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, parent=None):
        super(UndoableSetLineColorAlpha, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.cizgiRengi
        # if item.type() == shared.TEXT_ITEM_TYPE:
        #     self.eskiColor = item.defaultTextColor()
        # else:
        #     self.eskiColor = item._pen.color()
        #     if self.item.type() == shared.IMAGE_ITEM_TYPE:
        #         self.setText("change item's text alpha")

    # ---------------------------------------------------------------------
    def id(self):
        return 19

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.color = enSonUndo.color
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setCizgiRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.color, nesne_arkaplan_ikonu_guncelle=True)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setCizgiRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.eskiColor, nesne_arkaplan_ikonu_guncelle=True)


########################################################################
class UndoableSetTextColorAlpha(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, parent=None):
        super(UndoableSetTextColorAlpha, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.yaziRengi
        # if item.type() == shared.TEXT_ITEM_TYPE:
        #     self.eskiColor = item.defaultTextColor()
        # else:
        #     self.eskiColor = item._pen.color()
        #     if self.item.type() == shared.IMAGE_ITEM_TYPE:
        #         self.setText("change item's text alpha")

    # ---------------------------------------------------------------------
    def id(self):
        return 20

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.color = enSonUndo.color
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setYaziRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.color, nesne_arkaplan_ikonu_guncelle=True)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setYaziRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.eskiColor, nesne_arkaplan_ikonu_guncelle=True)


########################################################################
class UndoableSetItemBackgroundColor(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, renkSecicidenMi, parent=None):
        super(UndoableSetItemBackgroundColor, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.arkaPlanRengi
        self.renkSecicidenMi = renkSecicidenMi

    # # ---------------------------------------------------------------------
    # def id(self):
    #     return 21
    #
    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #     self.color = enSonUndo.color
    #     return True

    # # ---------------------------------------------------------------------
    # def mergeWithh(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #     if self.onizleme_mi:
    #         self.color = enSonUndo.color
    #         return True
    #     else:
    #         return False
    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setArkaPlanRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.color, self.item.yaziRengi,
                                                                           self.item.cizgiRengi, self.renkSecicidenMi)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setArkaPlanRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.eskiColor, self.item.yaziRengi,
                                                                           self.item.cizgiRengi, self.renkSecicidenMi)


########################################################################
class UndoableSetItemBackgroundColorAlpha(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, color, parent=None):
        super(UndoableSetItemBackgroundColorAlpha, self).__init__(description, parent)
        self.item = item
        self.color = color
        self.eskiColor = item.arkaPlanRengi
        if self.item.type() == shared.IMAGE_ITEM_TYPE:
            self.setText("change item's text background alpha")

    # ---------------------------------------------------------------------
    def id(self):
        return 22

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.color = enSonUndo.color
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setArkaPlanRengi(self.color)
        if self.item.isSelected():
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.color, self.item.yaziRengi,
                                                                           self.item.cizgiRengi)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setArkaPlanRengi(self.eskiColor)
        if self.item.isSelected():
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.eskiColor, self.item.yaziRengi,
                                                                           self.item.cizgiRengi)


########################################################################
class UndoableStilAdiDegistir(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, nesne, yeni_isim, parent=None):
        super(UndoableStilAdiDegistir, self).__init__(description, parent)

        self.nesne = nesne
        self.eski_isim = nesne.text()
        self.yeni_isim = yeni_isim

    # ---------------------------------------------------------------------
    def redo(self):
        self.nesne.setText(self.yeni_isim)

    # ---------------------------------------------------------------------
    def undo(self):
        self.nesne.setText(self.eski_isim)


########################################################################
class UndoableStiliUygula(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, eskiPen, yeniPen, yaziRengi, eskiYaziRengi, cizgiRengi, eskiCizgiRengi,
                 eskiArkaPlanRengi, yeniArkaPlanRengi, eskiFont, yeniFont,
                 scene, parent=None):
        super(UndoableStiliUygula, self).__init__(description, parent)
        self.font = yeniFont
        self.eskiFont = eskiFont
        self.pen = yeniPen
        self.eskiPen = eskiPen
        self.cizgiRengi = cizgiRengi
        self.eskiCizgiRengi = eskiCizgiRengi
        self.yaziRengi = yaziRengi
        self.eskiYaziRengi = eskiYaziRengi
        self.eskiArkaPlanRengi = eskiArkaPlanRengi
        self.yeniArkaPlanRengi = yeniArkaPlanRengi
        # self.cizgiKalinligi = yeniPen.widthF()
        # self.eskiCizgiKalinligi = eskiPen.widthF()
        self.scene = scene

    # # ---------------------------------------------------------------------
    # def id(self):
    #     return 23

    # # ---------------------------------------------------------------------
    # def mergeWith(self, enSonUndo):
    #     if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
    #         return False
    #     self.font = enSonUndo.font
    #     self.pen = enSonUndo.pen
    #     self.brush = enSonUndo.brush
    #     self.cizgiKalinligi = enSonUndo.cizgiKalinligi
    #     # self.eskiRotation = enSonUndo.eskiRotation
    #     # self.rotation += enSonUndo.rotation
    #     return True

    # ---------------------------------------------------------------------
    def redo(self):
        # self.scene.parent().currentFont=QFont(self.font)
        self.scene.aktifArac.yaziTipi = self.font

        self.scene.aktifArac.kalem = self.pen

        self.scene.aktifArac.yaziRengi = self.yaziRengi
        self.scene.aktifArac.cizgiRengi = self.cizgiRengi
        self.scene.aktifArac.arkaPlanRengi = self.yeniArkaPlanRengi

        self.scene.parent().degistir_yazi_rengi_ikonu(nesne_arkaplan_ikonu_guncelle=False)
        self.scene.parent().degistir_cizgi_rengi_ikonu(nesne_arkaplan_ikonu_guncelle=False)
        self.scene.parent().degistir_nesne_arkaplan_rengi_ikonu()
        self.scene.parent().change_font_point_sizef_spinbox_value(self.font.pointSizeF())
        self.scene.parent().change_line_style_options(self.pen)
        self.scene.parent().change_font_combobox_value(self.font)

    # ---------------------------------------------------------------------
    def undo(self):
        # self.scene.parent().currentFont=QFont(self.font)
        self.scene.aktifArac.yaziTipi = self.eskiFont

        self.scene.aktifArac.kalem = self.eskiPen

        self.scene.aktifArac.yaziRengi = self.eskiYaziRengi
        self.scene.aktifArac.cizgiRengi = self.eskiCizgiRengi
        self.scene.aktifArac.arkaPlanRengi = self.eskiArkaPlanRengi

        self.scene.parent().degistir_yazi_rengi_ikonu(nesne_arkaplan_ikonu_guncelle=False)
        self.scene.parent().degistir_cizgi_rengi_ikonu(nesne_arkaplan_ikonu_guncelle=False)
        self.scene.parent().degistir_nesne_arkaplan_rengi_ikonu()
        self.scene.parent().change_font_point_sizef_spinbox_value(self.eskiFont.pointSizeF())
        self.scene.parent().change_line_style_options(self.eskiPen)
        self.scene.parent().change_font_combobox_value(self.eskiFont)


########################################################################
class UndoableStiliNesneyeUygula(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, pen, brush, font, yaziRengi, cizgiRengi, parent=None):
        super(UndoableStiliNesneyeUygula, self).__init__(description, parent)
        self.item = item
        self.font = font
        self.eskiFont = item.font()
        self.pen = pen
        self.eskiPen = item.pen()
        self.brush = brush
        self.eskiBrush = item.brush()
        self.cizgiRengi = cizgiRengi
        self.eskiCizgiRengi = item.cizgiRengi
        self.yaziRengi = yaziRengi
        self.eskiYaziRengi = item.yaziRengi
        self.cizgiKalinligi = pen.widthF()
        self.eskiCizgiKalinligi = item._pen.widthF()

    # ---------------------------------------------------------------------
    def id(self):
        return 24

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.font = enSonUndo.font
        self.pen = enSonUndo.pen
        self.brush = enSonUndo.brush
        self.cizgiKalinligi = enSonUndo.cizgiKalinligi
        self.cizgiRengi = enSonUndo.cizgiRengi
        self.yaziRengi = enSonUndo.yaziRengi
        # self.eskiRotation = enSonUndo.eskiRotation
        # self.rotation += enSonUndo.rotation
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.setPen(self.pen)
        self.item.setCizgiRengi(self.cizgiRengi)
        # self.item.setYaziRengi(self.pen.color())
        self.item.setYaziRengi(self.yaziRengi)
        self.item.setCizgiKalinligi(self.cizgiKalinligi)  # cunku bi kac tane pen var guncellenmesi gereken
        # self.item.setBrush(self.brush)
        self.item.setArkaPlanRengi(self.brush.color())  # also sets brush
        self.item.setFont(self.font)
        # self.item.update()
        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.yaziRengi, nesne_arkaplan_ikonu_guncelle=False)
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.cizgiRengi, nesne_arkaplan_ikonu_guncelle=False)
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.brush.color(), self.yaziRengi,
                                                                           self.cizgiRengi)
            self.item.scene().parent().change_font_point_sizef_spinbox_value(self.font.pointSizeF())
            self.item.scene().parent().change_line_style_options(self.pen)
            self.item.scene().parent().change_font_combobox_value(self.font)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.setPen(self.eskiPen)
        self.item.setCizgiRengi(self.eskiCizgiRengi)
        self.item.setYaziRengi(self.eskiYaziRengi)
        self.item.setCizgiKalinligi(self.eskiCizgiKalinligi)  # cunku bi kac tane pen var guncellenmesi gereken
        # self.item.setBrush(self.eskiBrush)

        self.item.setArkaPlanRengi(self.eskiBrush.color())  # also sets brush
        self.item.setFont(self.eskiFont)

        if self.item.isSelected():
            self.item.scene().parent().degistir_yazi_rengi_ikonu(self.eskiYaziRengi,
                                                                 nesne_arkaplan_ikonu_guncelle=False)
            self.item.scene().parent().degistir_cizgi_rengi_ikonu(self.eskiCizgiRengi,
                                                                  nesne_arkaplan_ikonu_guncelle=False)
            self.item.scene().parent().degistir_nesne_arkaplan_rengi_ikonu(self.eskiBrush.color(), self.eskiYaziRengi,
                                                                           self.eskiCizgiRengi)
            self.item.scene().parent().change_font_point_sizef_spinbox_value(self.eskiFont.pointSizeF())
            self.item.scene().parent().change_line_style_options(self.eskiPen)
            self.item.scene().parent().change_font_combobox_value(self.eskiFont)


########################################################################
class UndoableSetImageOpacity(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, imageOpacity, parent=None):
        super(UndoableSetImageOpacity, self).__init__(description, parent)
        self.item = item
        self.imageOpacity = imageOpacity
        self.eskiImageOpacity = item.imageOpacity

    # ---------------------------------------------------------------------
    def id(self):
        return 25

    # ---------------------------------------------------------------------
    def mergeWith(self, enSonUndo):
        if not enSonUndo.id() == self.id() or self.item is not enSonUndo.item:
            return False
        self.imageOpacity = enSonUndo.imageOpacity
        return True

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.imageOpacity = self.imageOpacity
        self.item.update()

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.imageOpacity = self.eskiImageOpacity
        self.item.update()


########################################################################
class UndoableSetPinStatus(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, value, parent=None):
        """ """
        super(UndoableSetPinStatus, self).__init__(description, parent)

        self.item = item
        self.value = value

    # ---------------------------------------------------------------------
    def undo(self):
        # self.redo()
        self.item.isPinned = not self.value

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.isPinned = self.value


########################################################################
class UndoableItemSetText(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, eskiText, text, parent=None):
        """ """
        super(UndoableItemSetText, self).__init__(description, parent)

        self.item = item
        self.eski_text = eskiText
        self.text = text

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.scene().parent().act_aramayi_temizle()
        self.item.setText(self.text)

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.scene().parent().act_aramayi_temizle()
        self.item.setText(self.eski_text)


########################################################################
class UndoableConvertToPlainText(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, parent=None):
        """ """
        super(UndoableConvertToPlainText, self).__init__(description, parent)

        self.item = item
        self.html = item.toHtml()

    # ---------------------------------------------------------------------
    def redo(self):
        text = self.item.toPlainText()
        self.item.document().clear()
        self.item.setPlainText(text)
        self.item.isPlainText = True

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.document().clear()
        self.item.setHtml(self.html)
        self.item.isPlainText = False


########################################################################
class UndoableItemCustomCommand(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, command, parent=None):
        """ """
        super(UndoableItemCustomCommand, self).__init__(description, parent)

        self.item = item
        self.eski_command = item.command
        self.command = command

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.command = self.command
        self.item.setText(self.command["name"])

    # ---------------------------------------------------------------------
    def undo(self):
        self.item.command = self.eski_command
        self.item.setText(self.eski_command["name"])


########################################################################
class UndoableEmbedImage(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, parent=None):
        """ """
        super(UndoableEmbedImage, self).__init__(description, parent)

        self.item = item
        self.eskiImagePath = item.filePathForSave

        self.copyAlreadyExists = False
        for imgItem in self.item.scene().items():
            if imgItem.type() == shared.IMAGE_ITEM_TYPE:
                if imgItem.isEmbeded:
                    if self.item.originalSourceFilePath == imgItem.originalSourceFilePath:
                        self.yeniImagePath = imgItem.filePathForSave
                        self.copyAlreadyExists = True
                        break
        if not self.copyAlreadyExists:
            # self.yeniImagePath = item.scene().get_unique_path_for_embeded_image()
            self.yeniImagePath = item.scene().get_unique_path_for_embeded_image(os.path.basename(item.filePathForSave))

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.isEmbeded = True
        # TODO: bu copy2 hata dondurmuyor imis.
        # digerleri de metadata korumuyor, burda bi kontrol lazim aslinda.
        # eger kopyalandiysa degistirelim image adresi.
        if not self.copyAlreadyExists:
            shutil.copy2(self.item.filePathForSave, self.yeniImagePath)
        self.item.filePathForSave = self.yeniImagePath

    # ---------------------------------------------------------------------
    def undo(self):
        if not self.copyAlreadyExists:
            try:
                os.remove(self.yeniImagePath)
            except OSError as e:
                self.item.scene().parent().log(
                    'Warning! could not delete embeded image file, '
                    'but image\'s file path succesfully reverted to original path. ( {} )'.format(os.strerror(e.errno)),
                    20000, 3)
        # else:
        #     pass
        # finally:
        self.item.isEmbeded = False
        self.item.filePathForSave = self.eskiImagePath


########################################################################
class UndoableEmbedVideo(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, parent=None):
        """ """
        super(UndoableEmbedVideo, self).__init__(description, parent)

        self.item = item
        self.eskiVideoPath = item.filePathForSave

        self.copyAlreadyExists = False
        for videoItem in self.item.scene().items():
            if videoItem.type() == shared.VIDEO_ITEM_TYPE:
                if videoItem.isEmbeded:
                    if self.item.originalSourceFilePath == videoItem.originalSourceFilePath:
                        self.yeniVideoPath = videoItem.filePathForSave
                        self.copyAlreadyExists = True
                        break
        if not self.copyAlreadyExists:
            self.yeniVideoPath = item.scene().get_unique_path_for_embeded_video(os.path.basename(item.filePathForSave))

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.isEmbeded = True
        # TODO: bu copy2 hata dondurmuyor imis.
        # digerleri de metadata korumuyor, burda bi kontrol lazim aslinda.
        # eger kopyalandiysa degistirelim image adresi.
        if not self.copyAlreadyExists:
            shutil.copy2(self.item.filePathForSave, self.yeniVideoPath)
        self.item.filePathForSave = self.yeniVideoPath

    # ---------------------------------------------------------------------
    def undo(self):
        if not self.copyAlreadyExists:
            try:
                os.remove(self.yeniVideoPath)
            except OSError as e:
                self.item.scene().parent().log(
                    'Warning! could not delete embeded video file, '
                    'but video\'s file path succesfully reverted to original path. ( {} )'.format(os.strerror(e.errno)),
                    20000, 3)
        # else:
        #     pass
        # finally:
        self.item.isEmbeded = False
        self.item.filePathForSave = self.eskiVideoPath


########################################################################
class UndoableEmbedFile(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, item, parent=None):
        """ """
        super(UndoableEmbedFile, self).__init__(description, parent)

        self.item = item
        self.eskiFilePath = item.filePathForSave

        self.copyAlreadyExists = False
        for fileItem in self.item.scene().items():
            if fileItem.type() == shared.DOSYA_ITEM_TYPE:
                if fileItem.isEmbeded:
                    if self.item.originalSourceFilePath == fileItem.originalSourceFilePath:
                        self.yeniFilePath = fileItem.filePathForSave
                        self.copyAlreadyExists = True
                        break
        if not self.copyAlreadyExists:
            self.yeniFilePath = item.scene().get_unique_path_for_embeded_file(os.path.basename(item.filePathForSave))

    # ---------------------------------------------------------------------
    def redo(self):
        self.item.isEmbeded = True
        # TODO: bu copy2 hata dondurmuyor imis.
        # digerleri de metadata korumuyor, burda bi kontrol lazim aslinda.
        # eger kopyalandiysa degistirelim image adresi.
        if not self.copyAlreadyExists:
            shutil.copy2(self.item.filePathForSave, self.yeniFilePath)
        self.item.filePathForSave = self.yeniFilePath

    # ---------------------------------------------------------------------
    def undo(self):
        if not self.copyAlreadyExists:
            try:
                os.remove(self.yeniFilePath)
            except OSError as e:
                self.item.scene().parent().log(
                    'Warning! could not delete embeded file, '
                    'but file\'s file path succesfully reverted to original path. ( {} )'.format(os.strerror(e.errno)),
                    20000, 3)
        # else:
        #     pass
        # finally:
        self.item.isEmbeded = False
        self.item.filePathForSave = self.eskiFilePath


########################################################################
class UndoableSetSceneBackgroundBrush(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, view, color, parent=None):
        super(UndoableSetSceneBackgroundBrush, self).__init__(description, parent)
        self.view = view
        self.color = color
        self.eskiColor = self.view.backgroundBrush().color()

    # ---------------------------------------------------------------------
    def redo(self):
        self.view.setBackgroundBrush(self.color)

    # ---------------------------------------------------------------------
    def undo(self):
        self.view.setBackgroundBrush(self.eskiColor)


########################################################################
class UndoableSetSceneBackgroundImage(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, view, imagePath, parent=None):
        super(UndoableSetSceneBackgroundImage, self).__init__(description, parent)
        self.view = view
        self.imagePath = imagePath
        self.eskiImagePath = view.backgroundImagePath
        self.eskiEmbedDurumu = view.backgroundImagePathIsEmbeded

    # ---------------------------------------------------------------------
    def redo(self):
        self.view.set_background_image(self.imagePath)
        self.view.backgroundImagePathIsEmbeded = False

        # if not os.path.exists(self.imagePath):
        #     self.view.parent().log("Can not undo, %s does not exist or could not be read "
        #                             % self.imagePath, 5000, 2)

    # ---------------------------------------------------------------------
    def undo(self):
        self.view.set_background_image(self.eskiImagePath)
        self.view.backgroundImagePathIsEmbeded = self.eskiEmbedDurumu

        # if not os.path.exists(self.imagePath):
        #     self.view.parent().log("Can not undo, %s does not exist or could not be read "
        #                             % self.eskiImagePath, 5000, 2)


########################################################################
class UndoableEmbedSceneBackgroundImage(QUndoCommand):
    """ """

    # ---------------------------------------------------------------------
    def __init__(self, description, view, parent=None):
        """ """
        super(UndoableEmbedSceneBackgroundImage, self).__init__(description, parent)

        self.view = view
        self.eskiImagePath = view.backgroundImagePath

        self.kopya_var = False
        if os.path.exists(self.eskiImagePath):
            resim_klasor = os.path.join(self.view.scene().tempDirPath, "images")
            for resimAdi in os.listdir(resim_klasor):
                resimAdres = os.path.join(resim_klasor, resimAdi)
                if filecmp.cmp(view.backgroundImagePath, resimAdres, shallow=True):
                    self.kopya_var = True
                    break

        if self.kopya_var:
            self.yeniImagePath = resimAdres
        else:
            self.yeniImagePath = view.scene().get_unique_path_for_embeded_image(
                os.path.basename(view.backgroundImagePath))

    # ---------------------------------------------------------------------
    def redo(self):
        if not self.kopya_var:
            shutil.copy2(self.view.backgroundImagePath, self.yeniImagePath)
        self.view.set_background_image(self.yeniImagePath)
        self.view.backgroundImagePathIsEmbeded = True

    # ---------------------------------------------------------------------
    def undo(self):

        self.view.set_background_image(self.eskiImagePath)
        self.view.backgroundImagePathIsEmbeded = False
        if not self.kopya_var:
            try:
                os.remove(self.yeniImagePath)
            except OSError as e:
                self.view.parent().log(
                    'Warning! could not delete embeded background image file, '
                    'but scene\'s background image file path succesfully reverted to original path. ( {} )'.format(
                        os.strerror(e.errno)),
                    20000, 3)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '10/29/23'
__author__ = 'E. Y.'

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QSplitter, QFrame

from ..yanBolme.dikeyDugme import DikeyDugme
from ..yanBolme.kenarBoyut import KenarBoyutSol, KenarBoyutSag


# ########################################################################
# class Splitter(QSplitter):
#
#     # ---------------------------------------------------------------------
#     def __init__(self, orientation, parent=None):
#         super(Splitter, self).__init__(orientation, parent)
#
#         self.renkYazi = QColor(255, 255, 255)
#         self.renkArkaplan = QColor(200, 200, 200)
#
#     # ---------------------------------------------------------------------
#     def createHandle(self):
#         handle = super(Splitter, self).createHandle()
#         handle.setAutoFillBackground(True)
#
#         p = handle.palette()
#         p.setColor(handle.foregroundRole(), self.renkYazi)
#         p.setColor(handle.backgroundRole(), self.renkArkaplan)
#         handle.setPalette(p)
#
#         return handle


########################################################################
class YanBolme(QFrame):

    # ---------------------------------------------------------------------
    def __init__(self, solSag, parent=None):
        super(YanBolme, self).__init__(parent)

        self.setAcceptDrops(True)

        self.solSag = solSag

        self.setAutoFillBackground(True)

        p = self.palette()
        self.renkArkaplan = p.color(self.backgroundRole())
        self.renkArkaplanSurukleBirakBelirt = QColor(185, 255, 220)

        anaLay = QHBoxLayout(self)
        anaLay.setContentsMargins(0, 0, 0, 0)
        anaLay.setSpacing(0)

        self.dugmeLay = QVBoxLayout()
        self.dugmeLay.setContentsMargins(0, 0, 0, 0)
        self.dugmeLay.setSpacing(30)

        icerikLay = QHBoxLayout()
        icerikLay.setContentsMargins(0, 0, 0, 0)
        icerikLay.setSpacing(0)

        # -------
        self.icerikSplitter = QSplitter(Qt.Orientation.Vertical, self)
        self.icerikSplitter.setChildrenCollapsible(False)
        # self.icerikSplitter.setFixedWidth(genislik)
        # self.icerikSplitter.setFixedHeight(yukseklik-asagi_kayma)
        # self.icerikSplitter.splitterMoved.connect(self.splitter_boyut_degisti)

        if solSag == "sol":

            self.kenarBoyutWidget = KenarBoyutSol(splitterW=self.icerikSplitter,
                                                  ust_pencere_boyut=self.parent().size(),
                                                  renk=QColor(238, 238, 238), parent=self)
            self.kenarBoyutWidget.setMaximumWidth(5)
            self.kenarBoyutWidget.setMinimumWidth(5)
            self.kenarBoyutWidget.setMinimumHeight(50)
            # self.kenarBoyutWidget.hide()

            icerikLay.addWidget(self.icerikSplitter)
            icerikLay.addWidget(self.kenarBoyutWidget)

            anaLay.addLayout(self.dugmeLay)
            anaLay.addLayout(icerikLay)

        elif solSag == "sag":

            self.kenarBoyutWidget = KenarBoyutSag(splitterW=self.icerikSplitter,
                                                  ust_pencere_boyut=self.parent().size(),
                                                  renk=QColor(238, 238, 238), parent=self)
            self.kenarBoyutWidget.setMaximumWidth(5)
            self.kenarBoyutWidget.setMinimumWidth(5)
            self.kenarBoyutWidget.setMinimumHeight(50)
            # self.kenarBoyutWidget.hide()

            icerikLay.addWidget(self.kenarBoyutWidget)
            icerikLay.addWidget(self.icerikSplitter)

            anaLay.addLayout(icerikLay)
            anaLay.addLayout(self.dugmeLay)

    # # ---------------------------------------------------------------------
    # def splitter_boyut_degisti(self, pos, idx):
    #     print(pos, idx)

    # ---------------------------------------------------------------------
    def dugme_ve_yw_ekle(self, yazi, yw, yuklenirken_mi):

        # dugme = DikeyDugme(yazi, self, "yukari")
        dugme = DikeyDugme("", self, "yukari")
        # dugme.setToolTip(yazi)
        dugme.setStatusTip(yazi)
        dugme.setMaximumWidth(4)
        # dugme.setMaximumHeight(100)
        dugme.clicked.connect(self.ac_kapa)
        # dugme.btnYuzdur.clicked.connect(lambda: self.yw_yuzdur(dugme.yw))
        dugme.yw = yw
        yw.dugme = dugme
        self.dugmeLay.addWidget(dugme)

        if yuklenirken_mi:
            yw.cubuga_tasindi_yuklenirken(solSag=self.solSag)
            self.icerikSplitter.insertWidget(yw.sira, yw)
        else:
            yw.cubuga_tasindi(solSag=self.solSag)
            # self.icerikLay.addWidget(yw)
            yw.sira = self.icerikSplitter.count()
            self.icerikSplitter.addWidget(yw)
            yw.resize(yw.eskiSize)

        # self.kenarBoyutWidget.genislikleri_esitle()

        dugme.renk_degistir(acik_mi=not yw.kucuk_mu)

        # ywler hepsi yuzer haldeyken asagida kenarCubugu gizliyoruz
        # self.kenarBoyutWidget.show()

    # ---------------------------------------------------------------------
    def yw_yuzdur(self, yw):

        for i in range(self.dugmeLay.count()):
            dugme = self.dugmeLay.itemAt(i).widget()
            if dugme.yw is yw:
                self.dugmeLay.removeWidget(dugme)
                yw.dugme = None
                dugme.setParent(None)
                break
        yw.hide()
        yw.setParent(self.parent())
        yw.cubuktan_atildi()
        yw.show()
        # if not self.icerikSplitter.count():
        #     self.kenarBoyutWidget.hide()

        # idx = self.icerikSplitter.indexOf(yw)
        # yw2.setParent(self)

    # ---------------------------------------------------------------------
    def yw_cubuga_tasi_veya_kapat(self, yw, yuklenirken_mi):
        if yuklenirken_mi:
            self.dugme_ve_yw_ekle(yw.baslikEtiket.text(), yw, yuklenirken_mi)
        else:
            if yw.cubukta_mi:
                self.yw_yuzdur(yw)
            else:
                self.dugme_ve_yw_ekle(yw.baslikEtiket.text(), yw, yuklenirken_mi)

    # ---------------------------------------------------------------------
    def ac_kapa(self):
        dugme = self.sender()
        yw = dugme.yw
        if yw.isVisible():
            yw.kucult()
        else:
            yw.buyult()

    # ---------------------------------------------------------------------
    def renk_degistir(self, renkArkaplan=None):

        p = self.palette()
        # p.setColor(self.foregroundRole(), self.renkYazi)
        # ozellikle boyle yapildi, anlik widgeti baska renge boyayabiliyoruz
        # renk yoksa yine eski rengine don anlamında
        if renkArkaplan:
            p.setColor(self.backgroundRole(), renkArkaplan)
            self.kenarBoyutWidget.renk_degistir(renkArkaplan)
        else:
            # orjinal rengine don
            p.setColor(self.backgroundRole(), self.renkArkaplan)
            self.kenarBoyutWidget.renk_degistir()
        self.setPalette(p)

    # ---------------------------------------------------------------------
    def dragEnterEvent(self, e):
        e.accept()
        self.renk_degistir(self.renkArkaplanSurukleBirakBelirt)

    # ---------------------------------------------------------------------
    def dragLeaveEvent(self, e):
        e.accept()
        self.renk_degistir()

    # ---------------------------------------------------------------------
    def dropEvent(self, e):
        pos = e.pos()
        yw = e.source()
        dugme = yw.dugme

        dugmeAdet = self.dugmeLay.count()
        if not dugmeAdet:
            self.dugmeLay.addWidget(dugme)
            self.icerikSplitter.addWidget(yw)

        else:

            birak = False
            for n in range(dugmeAdet):
                w = self.dugmeLay.itemAt(n).widget()
                # birak = pos.y() < w.y() + w.size().height() // 2
                birak = pos.y() < w.y() + w.size().height()

                if birak:
                    if self.solSag == "sol":
                        yw.sola_yasla()
                    else:
                        yw.saga_yasla()

                    self.dugmeLay.insertWidget(n, dugme)
                    self.icerikSplitter.insertWidget(n, yw)
                    # self.dugmeTasindi.emit(dugme)
                    break

        e.accept()
        self.renk_degistir()

    # # ---------------------------------------------------------------------
    # def dropEventDugmeIcin(self, e):
    #     pos = e.pos()
    #     dugme = e.source()
    #     yw = dugme.yw
    #
    #     dugmeAdet = self.dugmeLay.count()
    #     if not dugmeAdet:
    #         self.dugmeLay.addWidget(dugme)
    #         self.icerikSplitter.addWidget(yw)
    #
    #     else:
    #
    #         birak = False
    #         for n in range(dugmeAdet):
    #             w = self.dugmeLay.itemAt(n).widget()
    #             birak = pos.y() < w.y() + w.size().height() // 2
    #
    #             if birak:
    #                 if self.solSag == "sol":
    #                     yw.sola_yasla()
    #                 else:
    #                     yw.saga_yasla()
    #
    #                 self.dugmeLay.insertWidget(n - 1, dugme)
    #                 self.icerikSplitter.insertWidget(n - 1, yw)
    #                 # self.dugmeTasindi.emit(dugme)
    #                 break
    #
    #     e.accept()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '18/Aug/2016'

from PySide6.QtCore import Signal, Slot
from PySide6.QtWidgets import QFontComboBox
from PySide6.QtGui import QFont


#######################################################################
class FontComboBox(QFontComboBox):
    valueChangedFromFontComboBoxGuiNotByCode = Signal(QFont)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(FontComboBox, self).__init__(parent)
        self.kodIleSetEdiliyor = False
        self.currentFontChanged.connect(self.on_current_font_changed)

    # ---------------------------------------------------------------------
    def setCurrentFont(self, font):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        QFontComboBox.setCurrentFont(self, font)
        self.kodIleSetEdiliyor = False

    # ---------------------------------------------------------------------
    @Slot(QFont)
    def on_current_font_changed(self, font):
        if not self.kodIleSetEdiliyor:
            self.valueChangedFromFontComboBoxGuiNotByCode.emit(font)
        # else:
        #     self.valueChangedWithSetValue.emit(val)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '5/5/16'

from PySide6.QtWidgets import QComboBox
from PySide6.QtCore import Signal, Slot


#######################################################################
class ComboBox(QComboBox):
    currentIndexChangedFromComboBoxGuiNotBySetCurrentIndex = Signal(int)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(ComboBox, self).__init__(parent)
        self.kodIleSetEdiliyor = False
        self.currentIndexChanged.connect(self.on_current_index_changed)

    # ---------------------------------------------------------------------
    def setCurrentIndex(self, idx):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        QComboBox.setCurrentIndex(self, idx)
        self.kodIleSetEdiliyor = False

    @Slot(int)
    def on_current_index_changed(self, idx):
        if not self.kodIleSetEdiliyor:
            self.currentIndexChangedFromComboBoxGuiNotBySetCurrentIndex.emit(idx)
        # else:
        #     self.valueChangedWithSetValue.emit(val)
